@@ -159,19 +159,22 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
       return None
 
     # check if all sample ids are valid
-    if not all([self._check_sample(item_id) for item_id in sample_ids]):
+    if not all((self._check_sample(item_id) for item_id in sample_ids)):
       return None
 
     data = {
       'controls': self._parsed_sample_list(control_ids, prefix),
       'samples': self._parsed_sample_list(sample_ids, prefix),
       'ignore_validation_warning': ignore_validation_warnings,
+      'meta': {'source': 'cli'},
       'workflow': '{}pipelines/{}'.format(prefix, workflow_id)
     }
 
     if project_id:
       data['projects'] = ['{}projects/{}'.format(prefix, project_id)]
-    eprint(json.dumps(data, indent=2))
+
+    if self.verbose:
+      eprint(json.dumps(data, indent=2))
 
     if params:
       data['params'] = params
@@ -277,9 +280,9 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     user_id = self._get_analysis_owner_id(analysis_id)
     return self.get_user(user_id) if user_id else None
 
-  def get_analyses(self):
+  def get_analyses(self, filters={}): # pylint: disable=dangerous-default-value
     '''Get resource list'''
-    return (Analysis(self.conf.get('api'))).list_all()
+    return (Analysis(self.conf.get('api'))).list_all(filters=filters)
 
   def update_analysis(self, uid, data):
     '''Update resource'''
@@ -289,7 +292,7 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     )
     if info.get('error'):
       eprint('couldn\'t update analysis {}, msg: {}'.format(uid, info.get('msg')))
-      return
+      return None
 
     if self.verbose:
       eprint('analysis', uid, 'updated')
@@ -375,9 +378,9 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
         cache='{}/json/genome.{}.json'.format(self.scratch, uid) if self.use_cache else False,
     )
 
-  def get_genomes(self):
+  def get_genomes(self, filters={}): # pylint: disable=dangerous-default-value
     '''Get resource list'''
-    return (Genome(self.conf.get('api'))).list_all()
+    return (Genome(self.conf.get('api'))).list_all(filters=filters)
 
   def update_genome(self, uid, data):
     '''Update resource'''
@@ -429,9 +432,9 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
         cache='{}/json/workflow.{}.json'.format(self.scratch, uid) if self.use_cache else False,
     )
 
-  def get_workflows(self):
+  def get_workflows(self, filters={}): # pylint: disable=dangerous-default-value
     '''Get resource list'''
-    return (Pipeline(self.conf.get('api'))).list_all()
+    return (Pipeline(self.conf.get('api'))).list_all(filters=filters)
 
   ################################################################################################
   ### PROJECT ####################################################################################
@@ -591,7 +594,7 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     info = (Sample(self.conf.get('api'))).delete(uid)
     if info.get('error'):
       eprint('error: deleting {}, msg: {}'.format(uid, info.get('msg')))
-      return
+      return None
 
     if self.verbose:
       eprint('deleted sample', uid)
@@ -612,20 +615,13 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     user_id = self._get_sample_owner_id(sample_id)
     return self.get_user(user_id) if user_id else None
 
-  def get_samples(self):
+  def get_samples(self, filters={}): # pylint: disable=dangerous-default-value
     '''Get resource list'''
-    return (Sample(self.conf.get('api'))).list_all()
+    return (Sample(self.conf.get('api'))).list_all(filters=filters)
 
-  def sample_name_to_id(self, name):
+  def samples_by_name(self, name, project_id=None):
     '''Get sample id from name'''
-    response = (Sample(self.conf.get('api'))).list({'limit': 2, 'name': name})
-    items = response.get('objects')
-    if not items:
-      eprint('WARNING: no sample by name', name)
-      return None
-    if len(items) > 1:
-      eprint('WARNING: multiple sample by name', name)
-    return items[0].get('id')
+    return (Sample(self.conf.get('api'))).by_name(name, project_id)
 
   def update_sample(self, uid, data):
     '''Update resource'''
@@ -687,10 +683,10 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
       cache='{}/json/upload.{}.json'.format(self.scratch, uid) if self.use_cache else False,
     )
 
-  def get_uploads(self):
+  def get_uploads(self, params={'limit': 0}): # pylint: disable=dangerous-default-value
     '''Get resource list'''
     info = (Upload(self.conf.get('api'))).list(
-        params={'limit': 0}
+        params=params
     )
     return info.get('objects', [])
 
@@ -893,7 +889,7 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     '''
     # some input checking
     if tags:
-      is_not_valid = not (isinstance(tags, list) and all([isinstance(item, list) for item in tags]))
+      is_not_valid = not (isinstance(tags, list) and all((isinstance(item, list) for item in tags)))
       if is_not_valid:
         eprint('Invalid tags argument. Provide a list of list of tags.')
         return None
@@ -1006,26 +1002,28 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
 
     if len(matches) > 1:
       matches.sort(
-        key=lambda item: datetime.datetime.strptime(item[1]['last_updated'], '%Y-%m-%dT%H:%M:%S.%f'),
+        key=lambda match: datetime.datetime.strptime(match[1]['last_updated'], '%Y-%m-%dT%H:%M:%S.%f'),
         reverse=True,
       )
       if not multiple:
         eprint('WARNING: multiple matching file for', node)
-      for file in matches:
-        eprint('\t', file[1]['last_updated'], file[1]['path'])
-
-    if multiple:
-      filepath = []
       for match in matches:
-        file1 = match[1]
-        path = self.download_file(file1['path'], dirname=dirname)
+        eprint('\t', match[1]['last_updated'], match[1]['path'])
+
+    filepath = []
+    for match in matches:
+      path = self.download_file(match[1]['path'], filename=dest, dirname=dest) if dest \
+        else self.download_file(match[1]['path'], dirname=dirname)
+
+      # if did download it then we added to the filepath else continue
+      if os.path.isfile(path):
         filepath.append(path)
+        if not multiple:
+          break
     else:
-      file1 = matches[0][1]
-      if dest:
-        dirname = dest
-      filepath = self.download_file(file1['path'], filename=dest, dirname=dirname)
-    return filepath
+      filepath.append(match[1]['path'])
+
+    return filepath if multiple else filepath[0]
 
   def get_file_by_tags(
     self,
@@ -1123,32 +1121,30 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
 
     if len(matches) > 1:
       matches.sort(
-        key=lambda item: datetime.datetime.strptime(item[1]['last_updated'], '%Y-%m-%dT%H:%M:%S.%f'),
+        key=lambda match: datetime.datetime.strptime(match[1]['last_updated'], '%Y-%m-%dT%H:%M:%S.%f'),
         reverse=True,
       )
       if not multiple:
         eprint('WARNING: multiple matching file for', tags)
-      for file in matches:
-        eprint('\t', file[1]['last_updated'], file[1]['path'])
-
-    if multiple:
-      filepath = []
       for match in matches:
-        file1 = match[1]
+        eprint('\t', match[1]['last_updated'], match[1]['path'])
 
-        if download:
-          path = self.download_file(file1['path'], dirname=dirname)
-          filepath.append(path)
-        else:
-          filepath.append(file1['path'])
-    else:
-      file1 = matches[0][1]
+
+    filepath = []
+    for match in matches:
       if download:
-        filepath = self.download_file(file1['path'], filename=dest, dirname=dirname)
+        path = self.download_file(match[1]['path'], filename=dest, dirname=dest) if dest \
+          else self.download_file(match[1]['path'], dirname=dirname)
+
+         # if did download it then we added to the filepath else continue
+        if os.path.isfile(path):
+          filepath.append(path)
+          if not multiple:
+            break
       else:
-        filepath = file1['path']
-        eprint('else')
-    return filepath
+        filepath.append(match[1]['path'])
+
+    return filepath if multiple else filepath[0]
 
   def get_filepath(self, filename, dirname=None):
     '''Use scratch and [dirname] to construct a full filepath'''
@@ -1212,7 +1208,7 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     parts = url.rsplit('/', 1)
     return {'id': parts[1]}
 
-  def print_data(self, data_type='', is_json=False, uid=None):
+  def print_data(self, data_type='', is_json=False, uid=None, project=None):
     '''
     Print data associated with genomes, samples, etc..
     Parameters
@@ -1220,6 +1216,7 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     data_type: {str}   Type of data to print (e.g. workflows)
     is_json:      {bool}  By default, data is printed in a human-readable format
     uid:       {list}  One or more ids of the objects you want
+    project: {int} project id to filter data
     '''
 
     if not isinstance(uid, list):
@@ -1254,13 +1251,20 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
         else:
           eprint('{} with uid {} invalid.'.format(data_type.capitalize(), item_id))
 
+    filters = {}
+    if project:
+      filters['projects__exact'] = project
+
     # if it is a list
     if data_type in list_methods:
-      data = getattr(self, list_methods.get(data_type))()
-
+      data = getattr(self, list_methods.get(data_type))(filters=filters)
 
     if not data:
-      eprint('Nothing found for the parameters you gave.')
+      eprint('No data found for the parameters you gave.')
+      return
+
+    if isinstance(data, dict) and data.get('error'):
+      eprint(data.get('msg', 'Error retrieving data.'))
       return
 
     # print the data as json
@@ -1281,6 +1285,11 @@ class BpApi(): # pylint: disable=too-many-instance-attributes,too-many-public-me
     analyses = [self.get_analysis(uid) for uid in analysis_ids]
     # remove null analyses, probably deleted or no ownership
     analyses = [analysis for analysis in analyses if not analysis.get('error')]
+    # sort them by latest updated
+    analyses.sort(
+      key=lambda analysis: datetime.datetime.strptime(analysis.get('last_updated'), '%Y-%m-%dT%H:%M:%S.%f'),
+      reverse=True,
+    )
     sample['analyses_full'] = analyses
     return sample
 
