@@ -6,5 +6,5 @@ from .abstract import Abstract
 class Host(Abstract):
   '''Webapp Host class'''
   def __init__(self, cfg):
-    super().__init__(cfg)
+    super(Host, self).__init__(cfg)
     self.endpoint += 'hosts/'
